'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var types = require('./types.js');

function useIdleCallback(callback, {
  unsupportedBehavior = types.UnsupportedBehavior.AnimationFrame
} = {}) {
  const handle = react.useRef(null);
  react.useEffect(() => {
    if ('requestIdleCallback' in window) {
      handle.current = window.requestIdleCallback(() => callback());
    } else if (unsupportedBehavior === types.UnsupportedBehavior.AnimationFrame) {
      handle.current = window.requestAnimationFrame(() => {
        callback();
      });
    } else {
      callback();
    }
    return () => {
      const {
        current: currentHandle
      } = handle;
      handle.current = null;
      if (currentHandle == null) {
        return;
      }
      if ('cancelIdleCallback' in window) {
        window.cancelIdleCallback(currentHandle);
      } else if (unsupportedBehavior === types.UnsupportedBehavior.AnimationFrame) {
        window.cancelAnimationFrame(currentHandle);
      }
    };
  }, [callback, unsupportedBehavior]);
}

exports.useIdleCallback = useIdleCallback;
